CraftBarData = {};

-- Bindings
BINDING_HEADER_CRAFTBARHEADER = "CraftBar";
BINDING_NAME_CRAFTBARSNAP = "MouseSnap";

local craftbar_professions = {
	['Tailoring'] = 3908,
	['Enchanting'] = 7411,
	['Disenchant'] = 13262,
	['Smelting'] = 2656,
	['Blacksmithing'] = 3100,
	['Alchemy'] = 2259,
	['Leatherworking'] = 2108,
	['Engineering'] = 4036,
	['Jewelcrafting'] = 25229,
	['Inscription'] = 45357,
	['Cooking'] = 2550,
	['First Aid'] = 3273,
}

local craftbar_buttons = {
	['Tailoring'] = nil,
	['Enchanting'] = nil,
	['Disenchant'] = nil,
	['Smelting'] = nil,
	['Blacksmithing'] = nil,
	['Alchemy'] = nil,
	['Leatherworking'] = nil,
	['Engineering'] = nil,
	['Jewelcrafting'] = nil,
	['Inscription'] = nil,
	['Cooking'] = nil,
	['First Aid'] = nil,
}

-- functions
function CraftBar_OnLoad()

 	DEFAULT_CHAT_FRAME:AddMessage("|c333399ffNirriti's |rCraftBar loaded.", 1.0, 1.0, 1.0, 1, 10);

 	this:RegisterEvent("ADDON_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");

	SLASH_RAIDICONBAR1 = "/craftbar";
  	SLASH_RAIDICONBAR2 = "/craft";
  	SlashCmdList["CRAFTBAR"] = function(msg)
		CraftBar_SlashCommandHandler(msg);
	end
end

function CraftBar_OnEvent(self, event, ...)
	if (event == "PLAYER_ENTERING_WORLD") then
	    CraftBar_initButtons();
	end
	if (event == "ADDON_LOADED") then
	
		local addonName = ...;
		if (addonName == "CraftBar") then
		    -- todo
		end
	end
	
	if (event == "PLAYER_ENTER_COMBAT" or event == "PLAYER_REGEN_DISABLED") then
	    local frame = getglobal("CraftBarFrame");
	    frame:Hide();
	end
end

function CraftBar_SlashCommandHandler(msg)
	if (msg == "reload") then
	 	ReloadUI();
	end
end

function CraftBar_initButtons()
 	
    CraftBarFrame:SetBackdrop( { 
        bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, 
        insets = { left = 4, right = 4, top = 4, bottom = 4 }
    });
    
    CraftBarFrame:Hide();

	for name, spellid in pairs(craftbar_professions) do 	    
 	    local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	
        local button = CreateFrame("Button", name, CraftBarFrame, "CraftBarButtonTemplate");
        
        button:RegisterForClicks("AnyUp");
        button.rangeTimer = -1;
	
        button:RegisterEvent("PLAYER_TARGET_CHANGED");
        button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

    	button:SetAttribute("type", "spell");
		button:SetAttribute("spell", spellname);
		button:SetAttribute("id", spellid);
			
		local icon = _G[button:GetName().."Icon"];
		icon:SetTexture(texture);
		
		craftbar_buttons[name] = button;
 	end
end

function CraftBar_initCraftFrame()	    

    local count = 0;

	for name, spellid in pairs(craftbar_professions) do 	    
 	    local spellname, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
 	    local usable, nomana = IsUsableSpell(spellname);
 	    
 	    local button = craftbar_buttons[name];

 	    if (usable == 1) then
 	    
            button:ClearAllPoints();
            button:SetPoint("LEFT", (8 + 40 * count), 0);
 	    
 	        button:Show();
 	        count = count + 1;
 	    else
 	        button:Hide();
 	    end
 	end
 	
    CraftBarFrame:SetWidth(10 + 40 * count);
end

function CraftBar_SnapDown()
	local frame = getglobal("CraftBarFrame");
	
	CraftBar_initCraftFrame();
	
	def_posx = frame:GetLeft();
	def_posy = frame:GetBottom();
	
	local x, y = GetCursorPosition();
	local scale = UIParent:GetEffectiveScale();
	
	x = x / scale / RaidIconBarData.scale;	
	y = y / scale / RaidIconBarData.scale;
	
	local frame = getglobal("CraftBarFrame");
	frame:ClearAllPoints();
	frame:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x, y);
	
	if (not InCombatLockdown()) then
	    frame:Show();
	end
end

function CraftBar_SnapUp()
	local frame = getglobal("CraftBarFrame");
	frame:ClearAllPoints();
	frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", def_posx, def_posy);
	
    if (not InCombatLockdown()) then
	    frame:Hide();
	end
end

function CraftBar_MouseOver(button)
	CraftBar_SetTooltip(button);
end

function CraftBar_MouseOut(button)
	GameTooltip:Hide();
	selectedButton = nil;
end

function CraftBar_SetTooltip(button)

    local spell = button:GetAttribute("spell");
    
    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    
    local spelltabid = CraftBar_FindSpellTabId(spell);    
    GameTooltip:SetSpell(spelltabid, BOOKTYPE_SPELL);
        
    GameTooltip:AppendText("");
end

function CraftBar_FindSpellTabId(spellname) 
    local spellCount = 0;
    
    for i = 1, MAX_SKILLLINE_TABS do 
        local name, texture, offset, numSpells = GetSpellTabInfo(i); 
        
        if not name then 
            break; 
        end 
        
        spellCount = spellCount + numSpells; 
    end
    
    for i=1, spellCount do 
        local spell, rank = GetSpellName(i, BOOKTYPE_SPELL); 
        
        if (spell) and (spellname == spell) then 
            return i; 
        end 
    end 
    
    return nil; 
end 